<?php

namespace App\Command;

use App\Entity\Client;
use App\Entity\User;
use App\Repository\UserRepository;
use App\Utility\UserUtility;
use App\Utility\Validator;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Security\Core\Encoder\UserPasswordEncoderInterface;
use Symfony\Contracts\Translation\TranslatorInterface;

class AddClientCommand extends Command
{
    protected static $defaultName = 'user:add';

    /**
     * @var SymfonyStyle
     */
    private $io;

    private $entityManager;
    private $passwordEncoder;
    private $translator;
    private $userRepository;
    private $validator;

    public function __construct(
        EntityManagerInterface $em,
        UserPasswordEncoderInterface $passwordEncoder,
        TranslatorInterface $translator,
        UserRepository $userRepository,
        Validator $validator
    ) {
        parent::__construct();

        $this->entityManager = $em;
        $this->passwordEncoder = $passwordEncoder;
        $this->translator = $translator;
        $this->userRepository = $userRepository;
        $this->validator = $validator;
    }

    protected function configure()
    {
        $this
            ->setDescription('Ajout d\'un nouvel utilisateur en base de données');
    }

    protected function initialize(InputInterface $input, OutputInterface $output): void
    {
        $this->io = new SymfonyStyle($input, $output);
    }

    protected function execute(
        InputInterface $input,
        OutputInterface $output
    ): int {
        $this->io->title("Ajout d'un utilisateur");

        $username = $this->io->ask(
            $this->translator->trans('addUserCmd.login'),
            null,
            [$this->validator, 'validateUsername']
        );

        $user = new User();
        $user->setUsername($username);

        $email = $this->io->ask(
            $this->translator->trans('addUserCmd.email'),
            null,
            [$this->validator, 'validateEmail']
        );

        $user->setEmail($email);

        $password = $this->io->askHidden(
            $this->translator->trans('addUserCmd.password'),
            [$this->validator, 'validatePassword']
        );

        $encodedPassword = $this->passwordEncoder->encodePassword($user, $password);
        $user->setPassword($encodedPassword);

        $confirmPassword = $this->io->askHidden(
            $this->translator->trans('addUserCmd.confirmPassword'),
            function ($confirmPassword) use ($password) {
                if (empty($confirmPassword)) {
                    throw new \RuntimeException($this->translator->trans('addUser.error.notEmpty'));
                }

                if ($confirmPassword !== $password) {
                    throw new \RuntimeException(
                        $this->translator->trans('addUser.error.confirmPasswordNoMatch')
                    );
                }

                return $confirmPassword;
            }
        );

        if ($password !== $confirmPassword) {
            $this->io->error($this->translator->trans('addUser.error.confirmPasswordNoMatch'));

            return 3;
        }

        $rolesList = [
            1 => $this->translator->trans('ROLE_NOTARY'),
            2 => $this->translator->trans('ROLE_AGENT'),
            3 => $this->translator->trans('ROLE_ADMIN'),
        ];
        $roleValue = $this->io->choice($this->translator->trans('addUserCmd.role'), $rolesList, 3);
        $roleKey = array_search($roleValue, $rolesList);
        $role = UserUtility::getRoleFromInt($roleKey);
        $user->setRoles([$role]);

        $name = $this->io->ask(
            $this->translator->trans('addUserCmd.name'),
            null,
            [$this->validator, 'validateNotEmpty']
        );
        $user->setName($name);

        if ($role === UserUtility::ROLE_AGENT) {
            $service = $this->io->ask(
                $this->translator->trans('addUserCmd.service'),
                null,
                [$this->validator, 'validateNotEmpty']
            );
            $user->setService($service);
        }

        if ($role === UserUtility::ROLE_NOTARY) {
            $address = $this->io->ask(
                $this->translator->trans('addUserCmd.address'),
                null,
                [$this->validator, 'validateNotEmpty']
            );
            $user->setAddress($address);

            $additionalAddress = $this->io->ask(
                $this->translator->trans('addUserCmd.additionnalAddress'),
                null,
                [$this->validator, 'validateNotEmpty']
            );
            $user->setAdditionalAddress($additionalAddress);

            $postalCode = $this->io->ask(
                $this->translator->trans('addUserCmd.postalCode'),
                null,
                [$this->validator, 'validateNotEmpty']
            );
            $user->setPostalCode($postalCode);

            $town = $this->io->ask(
                $this->translator->trans('addUserCmd.town'),
                null,
                [$this->validator, 'validateNotEmpty']
            );
            $user->setTown($town);
        }

        $this->entityManager->persist($user);
        $this->entityManager->flush();

        $this->io->success($this->translator->trans('addUser.success.userCreated', [$username, $role]));

        return 0;
    }
}
