<?php

namespace App\Controller;

use App\Data\SearchData;
use App\Entity\Instructor;
use App\Entity\Person;
use App\Entity\SearchLog;
use App\Entity\User;
use App\Form\SearchLogsFiltersType;
use App\Form\SearchType;
use App\Repository\PersonRepository;
use App\Repository\SearchLogRepository;
use App\Service\MailService;
use App\Service\SettingsDataService;
use App\Utility\MessageUtility;
use App\Utility\SearchUtility;
use App\Utility\UserUtility;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\Mime\Email;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Security\Core\Security;
use Symfony\Contracts\Translation\TranslatorInterface;

class SearchController extends BaseController
{
    protected $translator;

    /**
     * UserController constructor.
     *
     * @param SettingsDataService $settingsDataService
     * @param TranslatorInterface $translator
     */
    public function __construct(SettingsDataService $settingsDataService, TranslatorInterface $translator)
    {
        parent::__construct($settingsDataService);

        $this->translator = $translator;
    }

    /**
     * @Route("/search", name="search")
     *
     * @param Request     $request
     * @param MailService $mailService
     *
     * @return Response
     * @throws \Symfony\Component\Mailer\Exception\TransportExceptionInterface
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     */
    public function searchAction(Request $request, MailService $mailService)
    {
        $searchData = new SearchData();
        $persons = null;
        $searchStatus = 0;

        $form = $this->createForm(SearchType::class, $searchData);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $entityManager = $this->getDoctrine()->getManager();
            /** @var PersonRepository $personRepository */
            $personRepository = $entityManager->getRepository(Person::class);
            $useSoundex = $this->settingsData->hasSoundexSearch();
            $person = null;

            // *************************************************************************
            // Passe 1 : Recherche du nombre de réponses seulement avec les conditions :
            // - précise sur noms et prénoms avec suppression des traits d'union
            //     OU précise sur les noms et 3 premières lettres du prénom
            // - contrainte sur date de naissance complète
            // *************************************************************************
            $countPersons = $personRepository->countBySearch($searchData, $useSoundex);

            if ($countPersons > 1) {
                $searchStatus = SearchUtility::SEARCH_STATUS_AMBIGUOUS;
            } elseif ($countPersons === 1) {
                $persons = $personRepository->findBySearch($searchData, $useSoundex);
                $person = $persons[0];

                if ($person->getHelpCode() === SearchUtility::HELP_CODE_REC) {
                    // Si le code est 1SEXTREC alors il y a possibilité de récupération

                    $searchStatus = SearchUtility::SEARCH_STATUS_FIND_REC;
                } elseif ($person->getHelpCode() === SearchUtility::HELP_CODE_NOTREC) {
                    // Si le code est 1SEXTNONR alors il n'y a pas possibilité de récupération

                    $searchStatus = SearchUtility::SEARCH_STATUS_FIND_NOTREC;
                }
            } elseif ($countPersons === 0) {
                // *********************************************************************
                // Passe 2 : La requête est relancée sur l'année de naissance seulement
                // *********************************************************************

                $countPersons = $personRepository->countBySearch($searchData, $useSoundex, true);

                if ($countPersons >= 1) {
                    $searchStatus = SearchUtility::SEARCH_STATUS_AMBIGUOUS;
                } else {
                    // ******************************************************************************************
                    // Passe 3 : La requête est relancée sans les prénoms mais avec la date de naissance précise
                    // ******************************************************************************************

                    $countPersons = $personRepository->countBySearch($searchData, $useSoundex, false, true);

                    if ($countPersons >= 1) {
                        $searchStatus = SearchUtility::SEARCH_STATUS_AMBIGUOUS;
                    } else {
                        $searchStatus = SearchUtility::SEARCH_STATUS_NOTFIND;
                    }
                }
            }

            // ----------------------
            // REPONSE_INCONNU : élargissement aux demande possible dans plusieurs cas, le traitement est mutualisé ici
            // ----------------------
            if ($searchStatus === SearchUtility::SEARCH_STATUS_NOTFIND) {
                $countPersons = $personRepository->countBySearch($searchData, $useSoundex, false, false, true);

                if ($countPersons === 1) {
                    $this->sendMail(SearchUtility::SEARCH_STATUS_HAS_REQUEST, $mailService, $searchData);
                }
            }

            $this->addFlash(
                MessageUtility::MESSAGE_INFO,
                $this->translator->trans('searchResult.responseType.'.$searchStatus)
            );

            $validInstructors = [];

            if ($person) {
                $validInstructors = $this->findInstructorsForPerson($person, $searchStatus);
            }

            $this->sendMail($searchStatus, $mailService, $searchData, $validInstructors);

            $this->addSearchLog($searchData, $searchStatus, $person);

            return $this->redirectToRoute('search_result');
        }

        return $this->render(
            'search/search.html.twig',
            [
                'searchForm' => $form->createView(),
                'persons'    => $persons,
            ]
        );
    }

    /**
     * @Route("/search/result", name="search_result")
     *
     * @return Response
     */
    public function resultAction()
    {
        return $this->render('search/result.html.twig');
    }

    /**
     * @Route("/search_logs", name="search-logs")
     *
     * @param Request  $request
     * @param Security $security
     *
     * @return Response
     */
    public function logsAction(Request $request, Security $security)
    {
        /** @var User $user */
        $user = $this->getUser();

        $form = $this->createForm(SearchLogsFiltersType::class);
        $form->handleRequest($request);
        $searchLogs = null;

        if ($form->isSubmitted() && $form->isValid()) {
            $data = $form->getData();
            $entityManager = $this->getDoctrine()->getManager();
            /** @var SearchLogRepository $searchLogRepository */
            $searchLogRepository = $entityManager->getRepository(SearchLog::class);

            $currentUser = null;

            if (!$security->isGranted(UserUtility::ROLE_AGENT)) {
                $currentUser = $user;
            }

            $searchLogs = $searchLogRepository->findByFilter($data, $currentUser);
        }

        return $this->render(
            'search/logs.html.twig',
            [
                'logsFiltersForm' => $form->createView(),
                'searchLogs'      => $searchLogs,
            ]
        );
    }

    /**
     * @param SearchData  $searchData
     * @param int         $responseType
     * @param Person|null $person
     *
     * @throws \Exception
     */
    private function addSearchLog(SearchData $searchData, int $responseType, ?Person $person)
    {
        /** @var User $user */
        $user = $this->getUser();
        $entityManager = $this->getDoctrine()->getManager();

        $searchLog = $searchData->toSearchLog();
        $searchLog->setUser($user);
        $searchLog->setSearchDate(new \DateTime());
        $searchLog->setResponseType($responseType);
        if ($person) {
            $searchLog->setPerson($person->getPersonNum());
        }
        // TODO : A modifier quand le PDF sera généré
        $searchLog->setPdfLink('');

        $entityManager->persist($searchLog);
        $entityManager->flush();
    }

    /**
     * @param int         $searchStatus
     * @param MailService $mailService
     * @param SearchData  $searchData
     * @param array       $instructors
     *
     * @throws \Symfony\Component\Mailer\Exception\TransportExceptionInterface
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     */
    private function sendMail(
        int $searchStatus,
        MailService $mailService,
        SearchData $searchData,
        array $instructors = []
    ) {
        /** @var User $user */
        $user = $this->getUser();
        $mailSubject = $this->translator->trans(
            'searchResultMail.subject.default',
            [
                '%appName%'   => $this->settingsData->getAppName(),
                '%recipient%' => $searchData->getRecipient(),
                '%firstName%' => $searchData->getFirstName(),
                '%useName%'   => $searchData->getUseName(),
            ]
        );
        $mailTemplate = '';
        $mailParameters = [
            'firstName'       => $searchData->getFirstName(),
            'middleName'      => $searchData->getMiddleName(),
            'useName'         => $searchData->getUseName(),
            'civilName'       => $searchData->getCivilName(),
            'birthDate'       => $searchData->getBirthDate(),
            'deathDate'       => $searchData->getDeathDate(),
            'libelle_notaire' => $user->getName(),
            'mail_notaire'    => $user->getEmail(),
        ];
        $mailTo = $user->getEmail();

        switch ($searchStatus) {
            case SearchUtility::SEARCH_STATUS_FIND_REC:
            case SearchUtility::SEARCH_STATUS_FIND_NOTREC:
                $mailTemplate = 'emails/search_find.html.twig';
                break;
            case SearchUtility::SEARCH_STATUS_NOTFIND:
                $mailTemplate = 'emails/search_notfind.html.twig';
                break;
            case SearchUtility::SEARCH_STATUS_AMBIGUOUS:
                $mailTemplate = 'emails/search_ambiguous.html.twig';
                break;
            case SearchUtility::SEARCH_STATUS_HAS_REQUEST:
                $mailSubject = $this->translator->trans(
                    'searchResultMail.subject.hasRequest',
                    [
                        '%appName%' => $this->settingsData->getAppName(),
                    ]
                );
                $mailTemplate = 'emails/search_hasrequest.html.twig';
                // TODO : Envoie du mail au gestionnaire
                break;
        }

        /** @var User $user */
        $user = $this->getUser();

        $mailRecipients = [];
        $mailRecipients[] = $user->getEmail();

        // On envoie le mail aux instructeurs concernés
        if (!empty($instructors)) {
            /** @var Instructor $instructor */
            foreach ($instructors as $instructor) {
                $mailRecipients[] = $instructor->getEmail();
            }
        } else {
            $this->addFlash(
                MessageUtility::MESSAGE_WARNING,
                $this->translator->trans('searchResult.noInstructorFound')
            );
        }

        $mailHtml = $this->renderView(
            $mailTemplate,
            $mailParameters
        );

        $mailService->sendHtmlMail($mailRecipients, $mailSubject, $mailHtml);
    }

    /**
     * @param Person $person
     * @param int    $searchStatus
     *
     * @return array
     */
    private function findInstructorsForPerson(Person $person, int $searchStatus)
    {
        $entityManager = $this->getDoctrine()->getManager();
        $instructors = $entityManager->getRepository(Instructor::class)->findAll();

        $validInstructors = [];

        /** @var Instructor $instructor */
        foreach ($instructors as $instructor) {
            // Si l'instructeur est affecté pour ce type de réponse
            if (in_array($searchStatus, $instructor->getResponseType())) {
                // L'instructeur est configuré juste pour un Type de réponse (pas d'initiales)
                if (empty($instructor->getInitials())) {
                    $validInstructors[] = $instructor;

                    continue;
                }

                // On test si la personne commence par une des initiales affectées à l'instructeur
                foreach ($instructor->getInitials() as $initial) {
                    if (preg_match("/^[{$initial}]/i", $person->getUseName())) {
                        $validInstructors[] = $instructor;

                        break;
                    }
                }
            }
        }

        return $validInstructors;
    }
}
