<?php

namespace App\Controller;

use App\Entity\SearchLog;
use App\Entity\UserLog;
use App\Helper\GraphHelper;
use App\Repository\SearchLogRepository;
use App\Repository\UserLogRepository;
use App\Service\SettingsDataService;
use App\Utility\SearchUtility;
use DateTime;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Contracts\Translation\TranslatorInterface;

class StatsController extends BaseController
{
    /**
     * @var TranslatorInterface
     */
    protected $translator;

    /**
     * UserController constructor.
     *
     * @param SettingsDataService $settingsDataService
     * @param TranslatorInterface $translator
     */
    public function __construct(SettingsDataService $settingsDataService, TranslatorInterface $translator)
    {
        parent::__construct($settingsDataService);

        $this->translator = $translator;
    }

    /**
     * Affichage du tableau des statistiques
     *
     * @Route("/stats", name="stats")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     * @throws \Exception
     */
    public function statsAction(Request $request)
    {
        $dateTo = new DateTime();
        $dateFrom = new DateTime();
        $dateFrom->sub(new \DateInterval("P4M")); // affiche au maximum 5 mois
        $dateFrom->setDate(
            intval($dateFrom->format("Y")),
            intval($dateFrom->format("m")),
            1
        ); // force au premier jour du mois
        $dateFrom->setTime(0, 0); // force l'heure à 00:00

        [$stats, $tableHeader] = $this->getStats($dateFrom, $dateTo);

        $displayedStats = $this->verticalToHorizontalStatsArray($stats);

        return $this->render(
            'stats/stats.html.twig',
            [
                'tableHeader' => $tableHeader,
                'stats'       => $displayedStats,
            ]
        );
    }

    /**
     * Affichage du graph
     *
     * @Route("/stats/graph", name="stats_graph")
     */
    public function graphAction()
    {
        $dateTo = new DateTime();
        $dateFrom = new DateTime();
        $dateFrom->sub(new \DateInterval("P4M")); // affiche au maximum 5 mois
        $dateFrom->setDate(
            intval($dateFrom->format("Y")),
            intval($dateFrom->format("m")),
            1
        ); // force au premier jour du mois
        $dateFrom->setTime(0, 0); // force l'heure à 00:00

        [$stats] = $this->getStats($dateFrom, $dateTo);

        $graph = new GraphHelper($this->translator);
        $graph->render($dateTo, $dateFrom, $stats, false);
    }

    /**
     * @param DateTime $dateFrom
     * @param DateTime $dateTo
     *
     * @return array
     * @throws \Exception
     */
    private function getStats(DateTime $dateFrom, DateTime $dateTo): array
    {
        $entityManager = $this->getDoctrine()->getManager();
        /** @var SearchLogRepository $searchLogRepository */
        $searchLogRepository = $entityManager->getRepository(SearchLog::class);
        /** @var UserLogRepository $userLogRepository */
        $userLogRepository = $entityManager->getRepository(UserLog::class);

        $monthly = new \DateInterval('P1M');
        $stats = [];
        $monthsList = [];
        $iDate = clone $dateTo;

        // Initialisation du tableau de stats
        while ($iDate >= $dateFrom) {
            $monthsList[] = $iDate->format('U');
            $stats[$iDate->format('Y-m')] = [
                'nbRequests'        => 0,
                'nbResponses'       => 0,
                'nbResponsesByType' => [
                    SearchUtility::SEARCH_STATUS_FIND_REC    => 0,
                    SearchUtility::SEARCH_STATUS_FIND_NOTREC => 0,
                    SearchUtility::SEARCH_STATUS_AMBIGUOUS   => 0,
                    SearchUtility::SEARCH_STATUS_NOTFIND     => 0,
                ],
                'nbConnections'     => 0,
                'nbUniqueUsers'     => 0,
            ];

            $iDate->sub($monthly);
        }

        $searchStatsResult = $searchLogRepository->findSearchStats($dateFrom, $dateTo);

        // Remplissage du tableau de stats avec les données de recherche
        foreach ($searchStatsResult as $result) {
            if (!array_key_exists($result['month'], $stats)) {
                continue;
            }

            $curStats = &$stats[$result['month']];

            // Calcul du nombre total de réponses (filtre sur le mois + somme des valeurs de l'index 'nb')
            if ($curStats['nbResponses'] === 0) {
                $curMonthResults = array_filter(
                    $searchStatsResult,
                    function ($value) use ($result) {
                        return $value['month'] === $result['month'];
                    }
                );
                $curStats['nbResponses'] = array_sum(array_column($curMonthResults, 'nb'));
                $curStats['nbRequests'] = $curStats['nbResponses'];
            }

            $curStats['nbResponsesByType'][$result['responseType']] = $result['nb'];
        }

        // Nombre de connexions pour chaque mois
        $countLogResult = $userLogRepository->countLogByMonth($dateFrom, $dateTo);

        foreach ($countLogResult as $result) {
            if (!array_key_exists($result['month'], $stats)) {
                continue;
            }

            $curStats = &$stats[$result['month']];
            $curStats['nbConnections'] = $result['nb'];
        }

        // Nombre de connexions unique pour chaque mois
        $countUniqLogResult = $userLogRepository->countUniqLogByMonth($dateFrom, $dateTo);

        foreach ($countUniqLogResult as $result) {
            if (!array_key_exists($result['month'], $stats)) {
                continue;
            }

            $curStats = &$stats[$result['month']];
            $curStats['nbUniqueUsers'] = $result['nb'];
        }

        return [$stats, $monthsList];
    }

    /**
     * TODO : Peut-être trouver un truc plus "joli"
     *
     * @param array $stats
     *
     * @return array
     */
    private function verticalToHorizontalStatsArray(array $stats): array
    {
        $horizontalStats = [
            0 => [$this->translator->trans('stats.nbRequests')],
            1 => [$this->translator->trans('stats.nbResponses')],
            2 => [$this->translator->trans('stats.nbResponsesByType.'.SearchUtility::SEARCH_STATUS_FIND_REC)],
            3 => [$this->translator->trans('stats.nbResponsesByType.'.SearchUtility::SEARCH_STATUS_FIND_NOTREC)],
            4 => [$this->translator->trans('stats.nbResponsesByType.'.SearchUtility::SEARCH_STATUS_AMBIGUOUS)],
            5 => [$this->translator->trans('stats.nbResponsesByType.'.SearchUtility::SEARCH_STATUS_NOTFIND)],
            6 => [$this->translator->trans('stats.nbConnections')],
            7 => [$this->translator->trans('stats.nbUniqueUsers')],
        ];

        foreach ($stats as $monthStat) {
            $horizontalStats[0][] = $monthStat['nbRequests'];
            $horizontalStats[1][] = $monthStat['nbResponses'];
            $horizontalStats[2][] = $monthStat['nbResponsesByType'][SearchUtility::SEARCH_STATUS_FIND_REC];
            $horizontalStats[3][] = $monthStat['nbResponsesByType'][SearchUtility::SEARCH_STATUS_FIND_NOTREC];
            $horizontalStats[4][] = $monthStat['nbResponsesByType'][SearchUtility::SEARCH_STATUS_AMBIGUOUS];
            $horizontalStats[5][] = $monthStat['nbResponsesByType'][SearchUtility::SEARCH_STATUS_NOTFIND];
            $horizontalStats[6][] = $monthStat['nbConnections'];
            $horizontalStats[7][] = $monthStat['nbUniqueUsers'];
        }

        return $horizontalStats;
    }
}
