<?php

namespace App\Repository;

use App\Data\SearchData;
use App\Entity\Person;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\ORM\QueryBuilder;

/**
 * @method Person|null find($id, $lockMode = null, $lockVersion = null)
 * @method Person|null findOneBy(array $criteria, array $orderBy = null)
 * @method Person[]    findAll()
 * @method Person[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class PersonRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Person::class);
    }

    /**
     * @param SearchData $data
     * @param bool       $useSoundex
     * @param bool       $onlyBirthYear
     * @param bool       $withoutFirstName
     * @param bool       $onlyNullHelpCode
     *
     * @return int Nombre de résultats de recherche
     */
    public function countBySearch(
        SearchData $data,
        bool $useSoundex,
        bool $onlyBirthYear = false,
        bool $withoutFirstName = false,
        bool $onlyNullHelpCode = false
    ) {
        $queryBuilder = $this->createQueryBuilder('p');

        $queryBuilder->select('COUNT(DISTINCT p.id)');

        if ($useSoundex) {
            return $this->searchWithSoundex(
                $queryBuilder,
                $data,
                $onlyBirthYear,
                $withoutFirstName,
                $onlyNullHelpCode
            )->getSingleScalarResult();
        } else {
            return $this->search(
                $queryBuilder,
                $data,
                $onlyBirthYear,
                $withoutFirstName,
                $onlyNullHelpCode
            )->getSingleScalarResult();
        }
    }

    /**
     * @param SearchData $data
     *
     * @param bool       $useSoundex
     * @param bool       $onlyBirthYear
     * @param bool       $withoutFirstName
     * @param bool       $onlyNullHelpCode
     *
     * @return Person[]
     */
    public function findBySearch(
        SearchData $data,
        bool $useSoundex,
        bool $onlyBirthYear = false,
        bool $withoutFirstName = false,
        bool $onlyNullHelpCode = false
    ) {
        $queryBuilder = $this->createQueryBuilder('p');

        if ($useSoundex) {
            return $this->searchWithSoundex(
                $queryBuilder,
                $data,
                $onlyBirthYear,
                $withoutFirstName,
                $onlyNullHelpCode
            )->getResult();
        } else {
            return $this->search(
                $queryBuilder,
                $data,
                $onlyBirthYear,
                $withoutFirstName,
                $onlyNullHelpCode
            )->getResult();
        }
    }

    /**
     * @param QueryBuilder $queryBuilder
     * @param SearchData   $data
     * @param bool         $onlyBirthYear
     * @param bool         $withoutFirstName
     * @param bool         $onlyNullHelpCode
     *
     * @return mixed
     */
    private function searchWithSoundex(
        QueryBuilder &$queryBuilder,
        SearchData $data,
        bool $onlyBirthYear = false,
        bool $withoutFirstName = false,
        bool $onlyNullHelpCode = false
    ) {
        $parameters = [];

        $lastNames = $queryBuilder->expr()->orX(
            $queryBuilder->expr()->eq('SOUNDEX(p.useName)', 'SOUNDEX(:useName)'),
            $queryBuilder->expr()->eq('SOUNDEX(p.useName)', 'SOUNDEX(:civilName)'),
            $queryBuilder->expr()->eq('SOUNDEX(p.civilName)', 'SOUNDEX(:useName)'),
            $queryBuilder->expr()->eq('SOUNDEX(p.civilName)', 'SOUNDEX(:civilName)')
        );

        $parameters['useName'] = $data->getUseName();
        $parameters['civilName'] = $data->getCivilName();

        if ($withoutFirstName) {
            $orNames = $lastNames;
        } else {
            $orNames = $queryBuilder->expr()->orX(
                $queryBuilder->expr()->andX(
                    $lastNames,
                    $queryBuilder->expr()->orX(
                        $queryBuilder->expr()->eq('SOUNDEX(p.firstName)', 'SOUNDEX(:firstName)'),
                        $queryBuilder->expr()->eq('SOUNDEX(p.firstName)', 'SOUNDEX(:middleName)'),
                        $queryBuilder->expr()->eq('SOUNDEX(p.middleName)', 'SOUNDEX(:firstName)'),
                        $queryBuilder->expr()->eq('SOUNDEX(p.middleName)', 'SOUNDEX(:middleName)'),
                        $queryBuilder->expr()->eq('SOUNDEX(p.thirdName)', 'SOUNDEX(:firstName)'),
                        $queryBuilder->expr()->eq('SOUNDEX(p.thirdName)', 'SOUNDEX(:middleName)')
                    )
                ),
                $queryBuilder->expr()->andX(
                    $queryBuilder->expr()->orX(
                        $queryBuilder->expr()->eq('SOUNDEX(p.useName)', 'SOUNDEX(:useName)'),
                        $queryBuilder->expr()->eq('SOUNDEX(p.useName)', 'SOUNDEX(:civilName)'),
                        $queryBuilder->expr()->eq('SOUNDEX(p.civilName)', 'SOUNDEX(:useName)'),
                        $queryBuilder->expr()->eq('SOUNDEX(p.civilName)', 'SOUNDEX(:civilName)')
                    ),
                    $queryBuilder->expr()->like('p.firstName', ':firstNameStart')
                )
            );

            $parameters['firstName'] = $data->getFirstName();
            $parameters['middleName'] = $data->getMiddleName();
            $parameters['firstNameStart'] = $data->getFirstNameStart().'%';
        }

        if ($onlyBirthYear) {
            $birthDay = $queryBuilder->expr()->eq('YEAR(p.birthDate)', ':birthDate');
            $birthDayFormat = 'Y';
        } else {
            $birthDay = $queryBuilder->expr()->eq('p.birthDate', ':birthDate');
            $birthDayFormat = 'Y-m-d';
        }

        $parameters['birthDate'] = $data->getBirthDate()->format($birthDayFormat);

        if ($onlyNullHelpCode) {
            $queryBuilder->andWhere('p.helpCode IS NULL');
        } else {
            $queryBuilder->andWhere('p.helpCode IS NOT NULL');
        }

        return $queryBuilder
            ->andWhere($orNames)
            ->andWhere($birthDay)
            ->setParameters(
                $parameters
            )
            ->getQuery();
    }

    /**
     * @param QueryBuilder $queryBuilder
     * @param SearchData   $data
     * @param bool         $onlyBirthYear
     * @param bool         $withoutFirstName
     * @param bool         $onlyNullHelpCode
     *
     * @return mixed
     */
    private function search(
        QueryBuilder &$queryBuilder,
        SearchData $data,
        bool $onlyBirthYear = false,
        bool $withoutFirstName = false,
        bool $onlyNullHelpCode = false
    ) {
        $parameters = [];
        $lastNames = $queryBuilder->expr()->orX(
            $queryBuilder->expr()->in('p.useName', ':namesWhitoutHyphens'),
            $queryBuilder->expr()->in('p.civilName', ':namesWhitoutHyphens')
        );

        $parameters['namesWhitoutHyphens'] = $data->getAllNamesWithoutHyphens();

        if ($withoutFirstName) {
            $orNames = $lastNames;
        } else {
            $orNames = $queryBuilder->expr()->orX(
                $queryBuilder->expr()->andX(
                    $lastNames,
                    $queryBuilder->expr()->orX(
                        $queryBuilder->expr()->in('p.firstName', ':firstNames'),
                        $queryBuilder->expr()->in('p.middleName', ':firstNames'),
                        $queryBuilder->expr()->in('p.thirdName', ':firstNames')
                    )
                ),
                $queryBuilder->expr()->andX(
                    $queryBuilder->expr()->orX(
                        $queryBuilder->expr()->in('p.useName', ':names'),
                        $queryBuilder->expr()->in('p.civilName', ':names')
                    ),
                    $queryBuilder->expr()->like('p.firstName', ':firstNameStart')
                )
            );

            $parameters['names'] = $data->getAllNames();
            $parameters['firstNames'] = $data->getAllFirstNames();
            $parameters['firstNameStart'] = $data->getFirstNameStart().'%';
        }

        if ($onlyBirthYear) {
            $birthDay = $queryBuilder->expr()->eq('YEAR(p.birthDate)', ':birthDate');
            $birthDayFormat = 'Y';
        } else {
            $birthDay = $queryBuilder->expr()->eq('p.birthDate', ':birthDate');
            $birthDayFormat = 'Y-m-d';
        }

        $parameters['birthDate'] = $data->getBirthDate()->format($birthDayFormat);

        if ($onlyNullHelpCode) {
            $queryBuilder->andWhere('p.helpCode IS NULL');
        } else {
            $queryBuilder->andWhere('p.helpCode IS NOT NULL');
        }

        return $queryBuilder
            ->andWhere($orNames)
            ->andWhere($birthDay)
            ->setParameters(
                $parameters
            )
            ->getQuery();
    }
}
